import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.actuators.groups.*;

/**
An example demonstrating the framework ActuatorGroups using
test blocks for relative dragging and direct mapping.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class ActuatorGroups extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new ActuatorGroups();
	}
	
	// public interface =========================================
	
	public ActuatorGroups() {
		setTitle("Example - ActuatorGroups");
		
		new HelpDialog(this, "Help - ActuatorGroups",
		 "Example demonstrating the framework actuator groups " +
		 "using test blocks for relative dragging and direct " +
		 "mapping." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag to tilt and turn view." +
		 "\n" +
		 "> Use arrow keys-shift to tilt and turn view." +
		 "\n" +
		 "> Left-button drag or arrow keys to slide orange " +
		 "thing cumulatively." +
		 "\n" +
		 "> Right-button drag to rotate orange thing " +
		 "cumulatively about offset center." +
		 "\n" +
		 "> Middle-button drag to scale orange thing " +
		 "cumulatively about offset center." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "roll olive thing cumulatively." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup manipulation targets
		/// center affine target
		CenterAffineGroup affine = new CenterAffineGroup(
		 new TestThing(TestThing.BOX_DEFAULT,
		 TestThing.BALL_AFFINE));
		getWorld().addSceneNode(BasicBlocks.buildTarget(
		 affine, new Vector3d(0, 2, 0))[0]);
		
		//// center point at +Z tic
		affine.getCenter().initActuation(
		 new Vector4d(0, 0, TestThing.BALL_SIZE, 0));
		
		//// rotation Y-axis
		affine.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		 
		//// scale offset and limits
		affine.getScale().getPlugin().setSourceOffset(
		 new Vector4d(1, 1, 1, 0));
		affine.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.25, 4));

		/// sphere target
		RotationSphereGroup sphere =
		 new RotationSphereGroup(new TestThing(
		 TestThing.BOX_DEFAULT, TestThing.BALL_SPHERE));
		getWorld().addSceneNode(BasicBlocks.buildTarget(
		 sphere, new Vector3d(0, -2, 0))[0]);

		// setup manipulation controls
		InputDragMapper mapper;
		
		/// translation, first button
		mapper = ActuationBlocks.buildDirectMapper(
		 affine.getTranslation(), new Vector2d(.025, .025),
		 Mapper.DIM_X, Mapper.DIM_Y, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// rotation Y, second button
		mapper = ActuationBlocks.buildDirectMapper(
		 affine.getAxisAngle(), new Vector2d(.025, .025),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_SECOND,
		 Input.MODIFIER_NONE, Input.MODIFIER_IGNORE);
		
		/// scale, third button
		mapper = ActuationBlocks.buildDirectMapper(
		 affine.getScale(), new Vector2d(.025, .025),
		 Mapper.DIM_X, Mapper.DIM_Z, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_THIRD,
		 Input.MODIFIER_NONE, Input.MODIFIER_IGNORE);
		
		/// sphere, second + control
		InputDragSplitter splitter = new InputDragSplitter();
		
		mapper = ActuationBlocks.buildDirectMapper(
		 sphere.getRotationX(), new Vector2d(.025, -.025),
		 Mapper.DIM_NONE, Mapper.DIM_W, true);
		splitter.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 sphere.getRotationY(), new Vector2d(.025, .025),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		splitter.addEventTarget(mapper);
		 
		ActuationBlocks.buildRelativeDragger(splitter,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}