import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.actuators.*;

/**
Example demonstrating the framework actuators using direct
coordinate mapping.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class Actuators extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new Actuators();
	}
	
	// public interface =========================================
	
	public Actuators() {
		setTitle("Example - Actuators");
		
		new HelpDialog(this, "Help - Actuators",
		 "Example demonstrating the framework actuators using " +
		 "direct coordinate mapping." +
		 "\n" +
		 "\n" +
		 "> Left-button drag to slide red thing." +
		 "\n" +
		 "> Right-button drag to rotate green thing." +
		 "\n" +
		 "> Middle-button drag to scale blue thing." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		AppDisplay disp = getView().getDisplay();
		Group root = getWorld().getViewRoot();
		
		// setup views
		getWorld().addViewNode(
		 BasicBlocks.buildView(getView(),
		 new Vector3d(-4.6, 4.6, 11),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0)
		));
		
		// setup manipulation targets
		TransformGroup target[];
		
		/// translation target
		target = BasicBlocks.buildTarget(new TestThing(
		 TestThing.BOX_TRANSLATION, TestThing.BALL_DEFAULT),
		 new Vector3d(0, 3, 0));
		getWorld().addSceneNode(target[0]);
		
		Actuator xlt = new Actuator(new SVTransformGroupPlugin(
		 new TranslationPlugin(target[1])));
		
		/// rotation target
		target = BasicBlocks.buildTarget(new TestThing(
		 TestThing.BOX_ROTATION, TestThing.BALL_DEFAULT),
		 new Vector3d(0, 0, 0));
		getWorld().addSceneNode(target[0]);
		
		Actuator rotX = new Actuator(new MMTransformGroupPlugin(
		 new RotationPlugin(target[1], new Vector3d(1, 0, 0))));
		Actuator rotY = new Actuator(new MMTransformGroupPlugin(
		 new RotationPlugin(target[1], new Vector3d(0, 1, 0))));
		
		/// scale target
		target = BasicBlocks.buildTarget(new TestThing(
		 TestThing.BOX_SCALE, TestThing.BALL_DEFAULT),
		 new Vector3d(0, -3, 0));
		getWorld().addSceneNode(target[0]);
		
		Actuator scl = new Actuator(new SVTransformGroupPlugin(
		 new ScalePlugin(target[1])));
		scl.getPlugin().setTargetClamp(Mapper.DIM_ALL,
		 new Vector2d(.5, 4));

		// setup manipulation controls
		DirectInputDragPlugin plugin;
		InputDragMapper mapper;
		InputDragTarget filter;
		AwtMouseDragSensor dragger;
		
		/// relative cumulative translation, first button
		plugin = new DirectInputDragPlugin();
		plugin.setSourceScale(new Vector2d(.025, .025));
		mapper = new InputDragMapper(xlt, plugin);
		
		filter = new InputDragFilter(mapper,
		 new RelativeInputDragPlugin());
		 
		dragger = new AwtMouseDragSensor(disp, filter);
		dragger.setButtons(Input.BUTTON_FIRST);
		
		/// relative cumulative rotation, second button
		InputDragSplitter splitter = new InputDragSplitter();
		
		dragger = new AwtMouseDragSensor(disp, splitter);
		dragger.setButtons(Input.BUTTON_SECOND);

		//// X axis		
		plugin = new DirectInputDragPlugin();
		plugin.setSourceScale(new Vector2d(0, -.025));
		plugin.setSourceMap(Mapper.DIM_NONE, Mapper.DIM_W);
		mapper = new InputDragMapper(rotX, plugin);
		
		filter = new InputDragFilter(mapper,
		 new RelativeInputDragPlugin());
		splitter.addEventTarget(filter);

		//// Y axis
		plugin = new DirectInputDragPlugin();
		plugin.setSourceScale(new Vector2d(.025, 0));
		plugin.setSourceMap(Mapper.DIM_W, Mapper.DIM_NONE);
		mapper = new InputDragMapper(rotY, plugin);
		
		filter = new InputDragFilter(mapper,
		 new RelativeInputDragPlugin());
		splitter.addEventTarget(filter);
		
		/// relative non-cumulative scale, third button
		plugin = new DirectInputDragPlugin();
		plugin.setSourceScale(new Vector2d(.025, .025));
		plugin.setTargetOffset(new Vector4d(1, 1, 1, 0));
		mapper = new InputDragMapper(scl, plugin);
		mapper.setCumulative(false);
		
		filter = new InputDragFilter(mapper,
		 new RelativeInputDragPlugin());
		 
		dragger = new AwtMouseDragSensor(disp, filter);
		dragger.setButtons(Input.BUTTON_THIRD);

		// done
		showFrame();
	}
			
	// personal body ============================================

}