import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;
import j3dui.visualize.*;
import j3dui.visualize.change.*;

/**
Example demonstrating the constant sizing visualization
technique.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class ConstantSizing extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new ConstantSizing();
	}
	
	// public interface =========================================
	
	public ConstantSizing() {
		setTitle("Example - ConstantSizing");
		
		new HelpDialog(this, "Help - ConstantSizing",
		 "Example demonstrating the constant sizing " +
		 "visualization technique." +
		 "\n" +
		 "\n" +
		 "> Resize display window." +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag or arrow keys to turn " +
		 "side things." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "change display FOV." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Red thing remains constant size regardless of " +
		 "how thing, view, or display is manipulated." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// setup orbitable view, first + shift
		getView().setViewFov(Math.PI/2);
		
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(0, 0, 0), 5,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);

		// build sensors
		ViewChangeSplitter viewSpltr = new ViewChangeSplitter();
		new ViewExternalChangeSensor(viewSpltr, getView());
		new ViewInternalChangeSensor(viewSpltr, getView());

		// build world: normal center thing
		AffineGroup affine;
		Node thing;

		thing = new TestThing(TestThing.BOX_DISABLE_0,
		 TestThing.BALL_CANCEL);
		getWorld().addSceneNode(thing);

		// build world: side rotating things
		/// side thing rotator		
		AffineGroup affineY = new AffineGroup();
		getWorld().addSceneNode(affineY);
		
		/// side normal thing
		thing = new TestThing(TestThing.BOX_NORMAL_0,
		 TestThing.BALL_NORMAL);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(-3, 0, 0, 0));
		affineY.addNode(affine);

		/// side constant size thing
		double size = 0.0235;
		
		thing = new TestThing(TestThing.BOX_NORMAL_1,
		 TestThing.BALL_OVER);
		ConstantSizeGroup sizer = new ConstantSizeGroup(
		 thing, size);
		viewSpltr.addEventTarget(sizer);
		 
		affine = new AffineGroup(sizer);
		affine.getTranslation().initActuation(
		 new Vector4d(3, 0, 0, 0));
		affineY.addNode(affine);
		
		// setup manipulation controls
		InputDragTarget mapper;
		InputDragSplitter dragSpltr = new InputDragSplitter();
		
		/// side thing Y rotation, first + none
		mapper = ActuationBlocks.buildDirectMapper(
		 affineY.getAxisAngle(), new Vector2d(.01, -.01),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		affineY.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		dragSpltr.addEventTarget(mapper);
 
		ActuationBlocks.buildRelativeDragger(dragSpltr,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);

		/// view fov, first + control
		VisualizeBlocks.buildViewFover(getView(),
		 new Vector2d(Math.PI/4, Math.PI/1.5),
		 Input.BUTTON_FIRST, Input.MODIFIER_CTRL,
		 Input.MODIFIER_CTRL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}