import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;
import j3dui.feedback.elements.*;
import j3dui.visualize.*;
import j3dui.visualize.change.*;

/**
Example demonstrating the display overlay visualization
technique.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class DisplayOverlaying extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new DisplayOverlaying();
	}
	
	// public interface =========================================
	
	public DisplayOverlaying() {
		setTitle("Example - DisplayOverlaying");
		
		new HelpDialog(this, "Help - DisplayOverlaying",
		 "Example demonstrating the display overlay " +
		 "visualization technique." +
		 "\n" +
		 "\n" +
		 "> Resize display window." +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn a view." +
		 "\n" +
		 "> Left-button drag or arrow keys to turn display" +
		 "overlaid 3D thing." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "change display FOV; parallel projection at the " +
		 "limit." +
		 "\n" +
		 "\n" +
		 ">>> Display decorations remain overlaid and " +
		 "constant size regardless of how thing, view, or " +
		 "display is manipulated." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// setup orbitable view, first+shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, Math.PI/8, 0), 10,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// build sensors
		ViewChangeSplitter viewSpltr = new ViewChangeSplitter();
		new ViewInternalChangeSensor(viewSpltr, getView());

		// build world: normal center and side things
		AffineGroup affine;
		Node thing;

		thing = new TestThing(TestThing.BOX_DISABLE_0,
		 TestThing.BALL_CANCEL);
		getWorld().addSceneNode(thing);
		
		thing = new TestThing(TestThing.BOX_NORMAL_0,
		 TestThing.BALL_NORMAL);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(-3, 0, 0, 0));
		getWorld().addSceneNode(affine);

		thing = new TestThing(TestThing.BOX_NORMAL_0,
		 TestThing.BALL_NORMAL);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(3, 0, 0, 0));
		getWorld().addSceneNode(affine);
		
		// build display
		int zPlane;
		Vector2d origin = new Vector2d();
		Vector2d offset = new Vector2d();
		
		/// build display overlay root
		DisplayOverlayRoot dispRoot =
		 new DisplayOverlayRoot(getView());
		viewSpltr.addEventTarget(dispRoot);

		/// build lower-left control		
		origin.set(-.5, -.5);
		offset.set(50, 50);
				 
		DisplayOverlayGroup disp0 =
		 new DisplayOverlayGroup(dispRoot);
		viewSpltr.addEventTarget(disp0);
		disp0.setRelativeOrigin(origin);
		disp0.setAbsoluteOffset(offset);

		//// control mark
		zPlane = 1;
		
		thing = new TextureRect(true,
		 null, 0, Blocks.buildResourcePath(
		 "manipulate/NormalMark.gif"),
		 Elements.SIDE_TOP, 16, 16);
		 
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(32, 10, 0, 0));
		disp0.addNode(affine, zPlane);

		//// control back
		zPlane = 0;
		
		thing = new TextureRect(true,
		 null, 0, Blocks.buildResourcePath(
		 "manipulate/NormalRotateBack.gif"),
		 Elements.SIDE_TOP, 100, 100);
		disp0.addNode(thing, zPlane);

		//// control frame
		zPlane = -1;
		
		thing = new TextureRect(true,
		 new Color3f(.75f, .5f, .5f), .5, null,
		 Elements.SIDE_TOP, 100, 100);
		disp0.addNode(thing, zPlane);

		/// build lower-right control		
		origin.set(.5, -.5);
		offset.set(-50, 50);
				 
		DisplayOverlayGroup disp1 =
		 new DisplayOverlayGroup(dispRoot);
		viewSpltr.addEventTarget(disp1);
		disp1.setRelativeOrigin(origin);
		disp1.setAbsoluteOffset(offset);

		//// thing frontmost
		zPlane = 5;
		
		thing = new TestThing();
		 
		AffineGroup affineY = new AffineGroup(thing);
		affineY.getScale().initActuation(
		 new Vector4d(32, 32, 32, 0));
		disp1.addNode(affineY, zPlane);

		//// background backmost
		zPlane = -5;
		
		thing = new TextureRect(true,
		 new Color3f(0f, 1f, 0f), .5, null,
		 Elements.SIDE_TOP, 100, 100);
		disp1.addNode(thing, zPlane);
		
		// setup manipulation controls
		InputDragTarget mapper;
		InputDragSplitter dragSpltr = new InputDragSplitter();
		
		/// display thing Y rotation, first + none
		mapper = ActuationBlocks.buildDirectMapper(
		 affineY.getAxisAngle(), new Vector2d(.02, -.02),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		affineY.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		dragSpltr.addEventTarget(mapper);
 
		ActuationBlocks.buildRelativeDragger(dragSpltr,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);

		/// view fov, first + control
		VisualizeBlocks.buildViewFover(getView(),
		 new Vector2d(0, Math.PI/2), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}