import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.feedback.*;
import j3dui.manipulate.*;

/**
An example of an application using DnD amongst
Java3D and Swing components.
<P>
Refer to the book for further details about this example.
<P>
When using behaviors to sense input events a DnD drag started
outside of a Canvas3D display will freeze when the drag wanders
into the display.  For this reason there is no over feedback
or view orbiting.   Limited testing indicates that this does not
happen if AWT event detection is used.
<P>
A bug in Java 3D 1.2 causes the pieces of the model to
randomly appear and disappear when modifying the contents of
the scenegraph.  Run this example using Java 3D 1.1. 

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class DndApp extends DndFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new DndApp();
	}
	
	// public interface =========================================
	
	public DndApp() {
		setTitle("Example - DndApp");
		
		new HelpDialog(this, "Help - DndApp",
		 "An example of an application using DnD amongst " +
		 "Java3D and Swing components." +
		 "\n" +
		 "\n" +
		 "> Left-button drag items in the lists to lists or " +
		 "the view." +
		 "\n" +
		 "> Left-button drag items in the view to lists or " +
		 "the view." +
		 "\n" +
		 "\n" +
		 ">>> A bug in Java 3D 1.2 causes pieces of the model " +
		 "to randomly appear and disappear.  Run this example " +
		 "using Java 3D 1.1." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// build scene objects, add to target lists and scene
		Node object;
		Point3d min = new Point3d();
		Point3d max = new Point3d();
		ArrayList sourceList = new ArrayList();
		ArrayList targetList = new ArrayList();

		/// desk: in scene, draggable				
		object = ModelLoader.loadModelFromName(
		 Blocks.buildResourcePath("catalog/desk.wrl"));
		object = ManipulateBlocks.centerFloorTarget(object,
		 min, max, getWorld());
		PickUtils.setGeoPickable(object, true);
		object.setUserData(new DndNodeInfo("desk"));
		sourceList.add(object);
		targetList.add(object);
		getWorld().addSceneNode(object);
				
		/// lamp: not in scene, draggable				
		object = ModelLoader.loadModelFromName(
		 Blocks.buildResourcePath("catalog/lamp.wrl"));
		object = ManipulateBlocks.centerFloorTarget(object,
		 min, max, getWorld());
		PickUtils.setGeoPickable(object, true);
		object.setUserData(new DndNodeInfo("lamp"));
		sourceList.add(object);
		targetList.add(object);

		/// thing: not in scene, draggable				
		object = new TestThing();
		PickUtils.setGeoPickable(object, true);
		object = ManipulateBlocks.centerFloorTarget(object,
		 min, max, getWorld());
		object.setUserData(new DndNodeInfo("thing"));
		sourceList.add(object);
		targetList.add(object);

		/// floor: in scene, not draggable				
		object = new SecondBlocks.Room(new Vector3d(8, 3, 8));
		PickUtils.setGeoPickable(object, true);
		object.setUserData(new DndNodeInfo("floor"));
		targetList.add(object);
		getWorld().addSceneNode(object);
		
		// set DnD display (before accessing view or components)
		setDisplay(new DndAppDisplay(getWorld(), sourceList,
		 targetList, sourceList));

		// setup view
		getWorld().addViewNode(
		 BasicBlocks.buildView(getView(),
		 new Vector3d(0, 4, 8),
		 new Vector3d(-Math.PI/6, 0, 0)
		));
		
		// add objects to lists
		getListTop().addItem(
		 new DragSourceJLabel("lamp"));
		getListTop().addItem(
		 new DragSourceJLabel("desk"));
		getListTop().addItem(
		 new DragSourceJLabel("thing"));
		
		getListBottom().addItem(
		 new DragSourceJLabel("thing"));
		getListBottom().addItem(
		 new DragSourceJLabel("lamp"));

		// done
		showFrame();
	}
			
	// personal body ============================================

}