import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.Blocks;

/**
Dnd application frame containing an AppWorld, an AppView with
a DnD display, and two DnD lists with DnD elements.  The 2D UI
components are implemented using Swing components.

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class DndFrame extends JFrame {
	
	// public interface =========================================
	
	/**
	Constructs a DndFrame with an empty virtual world.  Have
	to add the display with setDisplay() before using the frame
	view.
	*/
	public DndFrame() {
		// kill the window on close
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent winEvent) {
				System.exit(0);
			}
		});
		
		_world = new AppWorld();
	}
	
	/**
	Build the view with the specified display, finish the frame.
	*/
	public void setDisplay(AppDisplay display) {
		// build view with display
	 	_view = new AppView(display);
	 	
		// tell Swing to use heavyweight popups	 
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		ToolTipManager.sharedInstance().
		 setLightWeightPopupEnabled(false);
				 		
		// build Swing window components
		buildContentPanel();
	}

	/**
	Makes the world live, sizes and shows the frame.
	*/
	public void showFrame() {
		_world.setLive(true);
		
		// size contents THEN show frame
		pack();
		show();
	}

	/**
	Gets the frame's 3D world.
	@return The 3D world.
	*/
	public AppWorld getWorld() {
		return _world;
	}

	/**
	Gets the frame's view.
	@return The left view.
	*/
	public AppView getView() {
		return _view;
	}

	/**
	Gets the frame's top data list.
	@return The data list.
	*/
	public DropTargetJPanel getListTop() {
		return _listTop;
	}

	/**
	Gets the frame's bottom data list.
	@return The data list.
	*/
	public DropTargetJPanel getListBottom() {
		return _listBtm;
	}
			
	// personal body ============================================

	/** virtual world. */
	private AppWorld _world;

	/** window view. */
	private AppView _view = null;

	/** top data list. */
	private DropTargetJPanel _listTop;

	/** bottom data list. */
	private DropTargetJPanel _listBtm;
				
	/**
	Builds the window contents panel, which consists of a
	view panel, a data list panel, and adds it to the window.
	*/
	protected void buildContentPanel() {
		// build main panel
		JSplitPane panel = new JSplitPane(
		 JSplitPane.HORIZONTAL_SPLIT);
		panel.setDoubleBuffered(true);
		panel.setContinuousLayout(true);
		panel.setOneTouchExpandable(true);

		// build view subpanel
		panel.setLeftComponent(_view.getDisplay());
		panel.getLeftComponent().setSize(
		 new Dimension(300, 300));
		
		// build data subpanel
		JSplitPane dataPanel = new JSplitPane(
		 JSplitPane.VERTICAL_SPLIT);
		panel.setRightComponent(dataPanel);
		
		dataPanel.setPreferredSize(new Dimension(100, 300));
		dataPanel.setDoubleBuffered(true);
		dataPanel.setContinuousLayout(true);
		dataPanel.setOneTouchExpandable(true);

		/// build scrollable droppable list		
		_listTop = new DropTargetJPanel();
		dataPanel.setTopComponent(new JScrollPane(_listTop));
		
		_listBtm = new DropTargetJPanel();
		dataPanel.setBottomComponent(new JScrollPane(_listBtm));

		// add main panel to frame		
		getContentPane().add(panel, BorderLayout.CENTER);
	}
	
}