import java.awt.*;
import java.awt.dnd.*;

import j3dui.utils.Debug;

/**
Contains constants and utilities of general use for drag
and drop operations.

<B>DnD Notes:</B>
The basic DnD actions are: Move, Copy, and Link,
with corresponding ACTION_??? flags in DnDConstants.
<P>
The "source" and "target" of a DnD action support
none or more of these actions, which are specified
when the source and target DnD objects are created.
During a DnD operation the user specifies one of
the DnD actions with the drag gesture, which can be
changed on the fly using modifier keys.
<P>
The drag cursor reflects the user action (move,
copy, link) and the droppability (drop action) of
the source on the target (drop, no drop).  The
drop action is determined from the intersection of
the source, target, and user actions.  If the result
is no actions then the cursor will appear as "no drop"
and any drop should be rejected by the target
(event.rejectDrop()).
<P>
In Windows, the modifier keys and their action are:
no keys for Move, Control key for Copy, and Control+Shift
keys for Link.  Also, Windows does not support
a drag "image".

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class DndUtils {
		
	// public utilities ========================================================

	// misc
	public static String toString(Point v) {
		return new String("[" +
		 v.x +
		 " " +
		 v.y +
		 "]");
	}

	public static String toString(Image v) {
		return new String("[" +
		 "Image:" +
		 " Src=" + v.getSource() +
		 "]");
	}

	// gesture
	public static String toString(DragGestureEvent v) {
		return new String("[" +
		 "DragSourceEvent:" +
		 " Cmp=" + v.getComponent() +
		 " DrgAct=" + v.getDragAction() +
		 " DrgOrg=" + toString(v.getDragOrigin()) +
		 " DrgSrc=" + toString(v.getDragSource()) +
		 " DrgGstRnz=" + v.getSourceAsDragGestureRecognizer() +
		 " TrgEvt=" + v.getTriggerEvent() +
		 "]");
	}
	
	// source
	public static String toString(DragSource v) {
		return new String("[" +
		 "DragSource:" +
		 v.toString() +
		 "]");
	}

	public static String toString(DragSourceContext v) {
		return new String("[" +
		 "DragSourceContext:" +
		 " Cmp=" + v.getComponent() +
		 " Cur=" + v.getCursor() +
		 " DrgSrc=" + toString(v.getDragSource()) +
		 " SrcAct=" + v.getSourceActions() +
		 " Trg=" + v.getTrigger() +
		 "]");
	}
	
	public static String toString(DragSourceEvent v) {
		return new String("[" +
		 "DragSourceEvent:" +
		 " DrgSrcCtx=" + toString(v.getDragSourceContext()) +
		 "]");
	}

	public static String toString(DragSourceDragEvent v) {
		return new String("[" +
		 "DragSourceDragEvent:" +
		 " DrpAct=" + v.getDropAction() +
		 " GstMod=" + v.getGestureModifiers() +
		 " TrgAct=" + v.getTargetActions() +
		 " UsrAct=" + v.getUserAction() +
		 " DrgSrcCtx=" + toString(v.getDragSourceContext()) +
		 "]");
	}

	public static String toString(DragSourceDropEvent v) {
		return new String("[" +
		 "DragSourceDropEvent:" +
		 " DrpAct=" + v.getDropAction() +
		 " DrpSuc=" + v.getDropSuccess() +
		 " DrgSrcCtx=" + toString(v.getDragSourceContext()) +
		 "]");
	}

	// target
	public static String toString(DropTarget v) {
		return new String("[" +
		 "DropTarget:" +
		 " Cmp=" + v.getComponent() +
		 " DefAct=" + v.getDefaultActions() +
		 " DrpTrgCtx=" + v.getDropTargetContext() +
		 "]");
	}

	public static String toString(DropTargetContext v) {

// bug test
Component x = v.getComponent();
// The following toString() causes the bug, not simply
// getting the component.
//x.toString();

		return new String("[" +
		 "DropTargetContext:" +
// bug?: using toString on anything in context screws up DnD
//		 " Cmp=" + v.getComponent() +
//		 " DrpTrg=" + toString(v.getDropTarget()) +
v +
		 "]");
	}

	public static String toString(DropTargetEvent v) {
		return new String("[" +
		 "DropTargetEvent:" +
		 " DrpTrgCtx=" + toString(v.getDropTargetContext()) +
		 "]");
	}

	public static String toString(DropTargetDragEvent v) {
		return new String("[" +
		 "DropTargetDragEvent:" +
		 " DrpAct=" + v.getDropAction() +
		 " Loc=" + toString(v.getLocation()) +
		 " SrcAct=" + v.getSourceActions() +
		 " DrpTrgCtx=" + toString(v.getDropTargetContext()) +
		 "]");
	}

	public static String toString(DropTargetDropEvent v) {
		return new String("[" +
		 "DropTargetDropEvent:" +
		 " DrpAct=" + v.getDropAction() +
		 " Loc=" + toString(v.getLocation()) +
		 " SrcAct=" + v.getSourceActions() +
		 " DrpTrgCtx=" + toString(v.getDropTargetContext()) +
		 "]");
	}
			
	// personal body ===========================================================
	
}
	