import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;

import j3dui.utils.Debug;

/**
A JLabel capable of initiating drag operations.

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class DragSourceJLabel extends JLabel 
 implements DragSourceListener, DragGestureListener {
	
	// public interface =========================================
	
	/**
	Construct a DragSourceJLabel with default drag actions.
	*/
	public DragSourceJLabel(String text) {
		this(text, DnDConstants.ACTION_COPY_OR_MOVE |
		 DnDConstants.ACTION_LINK);
	}

	/**
	Construct a DragSourceJLabel with the specified drag actions.
	@param actions Flags specifying the types of drag actions
	this object will provide (DnDConstants.ACTION_???).
	*/
	public DragSourceJLabel(String text, int actions) {
		super(text, CENTER);
		_dragSource = new DragSource();
		_dragGesture = _dragSource.
		 createDefaultDragGestureRecognizer(this, actions, this);
	}

	// implements java.awt.dnd.DragGestureListener
	 
	public void dragGestureRecognized(DragGestureEvent evt) {
	
if(Debug.getEnabled()){
Debug.println(this, "DragSourceJLabel.gesture",
"GESTURE:dragGestureRecognized:" +
" event=" + DndUtils.toString(evt));}

		// use label name as transfer data, start drag
		StringSelection item = new StringSelection(getText());
		
		_dragSource.startDrag(
		 evt, DragSource.DefaultCopyNoDrop, item, this);
	}

	// implements  java.awt.dnd.DragSourceListener
	 
	public void dragDropEnd(DragSourceDropEvent evt) {}

	public void dragEnter(DragSourceDragEvent evt) {}

	public void dragExit(DragSourceEvent evt) {}

	public void dragOver(DragSourceDragEvent evt) {
	
if(Debug.getEnabled()){
Debug.println(this, "DragSourceJLabel.source",
"SOURCE:dragOver:" +
" event=" + DndUtils.toString(evt));}

	}

	public void dropActionChanged(DragSourceDragEvent evt) {}
			
	// personal body ============================================

	private DragSource _dragSource;
	private DragGestureRecognizer _dragGesture;
	
}