import java.util.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.io.IOException;
import javax.swing.*;

import j3dui.utils.Debug;

/**
A JPanel capable of accepting drop operations.

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class DropTargetJPanel extends JPanel
 implements DropTargetListener {
	
	// public interface =========================================

	/**
	Construct a DropTargetJPanel with default drop actions.
	*/
	public DropTargetJPanel() {
		this(DnDConstants.ACTION_COPY_OR_MOVE |
		 DnDConstants.ACTION_LINK);
	}

	/**
	Construct a DropTargetJPanel with the specified drop actions.
	@param actions Flags specifying the types of drop actions
	this object will accept (DnDConstants.ACTION_???).
	*/
	public DropTargetJPanel(int actions) {
		_dropTarget = new DropTarget(this, actions, this);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	}

	/**
	Adds a component item to this panel and updates its
	presentation.
	@param item Item to be added.
	*/
	public void addItem(Component item) {
		add(item);
		revalidate();
	}

	/**
	Removes a component item from this panel and updates its
	presentation.
	@param item Item to be removed.
	*/
	public void removeItem(Component item) {
		remove(item);
		revalidate();
	}
	
	// DropTargetListener implementation
	
	public void dragEnter (DropTargetDragEvent evt) {}

	public void dragExit (DropTargetEvent evt) {}

	public void dragOver(DropTargetDragEvent evt) {
	
if(Debug.getEnabled()){
Debug.println(this, "DropTargetJPanel.target",
"TARGET:dragOver:" +
" event=" + DndUtils.toString(evt));}

	}

	public void drop(DropTargetDropEvent evt) {
	
if(Debug.getEnabled()){
Debug.println(this, "DropTargetJPanel.drop",
"TARGET:drop:" +
" event=" + DndUtils.toString(evt));}

		// if unsupported action, no drop
		if(evt.getDropAction()==DnDConstants.ACTION_NONE) {
		
if(Debug.getEnabled()){
Debug.println(this, "DropTargetJPanel.drop",
"TARGET:drop:" +
" No action.");}

			evt.rejectDrop();
			return;
		}
		
		// if unsupported transfer data, no drop
		Transferable tr = evt.getTransferable();
		if(!tr.isDataFlavorSupported(DataFlavor.stringFlavor)) {
		
if(Debug.getEnabled()){
Debug.println(this, "DropTargetJPanel.drop",
"TARGET:drop:" +
" Bad flavor.");}

			evt.rejectDrop();
			return;
		}

		// good drop, do it		
		try {
			evt.acceptDrop(evt.getDropAction());
			
			// user transfer data as drop source object name
			String item = (String)tr.getTransferData(
			 DataFlavor.stringFlavor);
				
if(Debug.getEnabled()){
Debug.println(this, "DropTargetJPanel.drop",
"TARGET:drop:" +
" item=" + item);}

			// add source object to this list
			addItem(new DragSourceJLabel(item));
			
			// done with drop			
			evt.getDropTargetContext().dropComplete(true);
		} catch (IOException io) {
			io.printStackTrace();
			evt.rejectDrop();
		} catch (UnsupportedFlavorException ufe) {
			ufe.printStackTrace();
			evt.rejectDrop();
		}
	}

	public void dropActionChanged (DropTargetDragEvent evt) {}
			
	// personal body ============================================

	private DropTarget _dropTarget;

 }