import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;

/**
Example demonstrating DRM (display-relative) intuitive
control mapping.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class DrmMapping extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new DrmMapping();
	}
	
	// public interface =========================================
	
	public DrmMapping() {
		setTitle("Example - DrmMapping");
		
		new HelpDialog(this, "Help - DrmMapping",
		 "Example demonstrating DRM (display-relative) " +
		 "intuitive control mapping." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag red thing to slide it." +
		 "\n" +
		 "> Left-button drag green thing to rotate it." +
		 "\n" +
		 "> Left-button drag blue thing to scale it." +
		 "\n" +
		 "> Left-button drag magenta thing to roll it." +
		 "\n" +
		 "> Use arrow keys with mouse over thing to " +
		 "manipulate it." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Tilt and turn view, then repeat manipulations." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup manipulation targets
		/// translation target
		AffineGroup affineXlt = new AffineGroup(new TestThing(
		 TestThing.BOX_TRANSLATION, TestThing.BALL_DEFAULT));
		 
		affineXlt.getTranslation().initActuation(
		 new Vector4d(0, 3, 0, 0));
		getWorld().addSceneNode(affineXlt);
		
		/// rotation target
		AffineGroup affineRot = new AffineGroup(new TestThing(
		 TestThing.BOX_ROTATION, TestThing.BALL_DEFAULT));
		 
		affineRot.getTranslation().initActuation(
		 new Vector4d(0, 0, 0, 0));
		getWorld().addSceneNode(affineRot);
		
		/// scale target
		AffineGroup affineScl = new AffineGroup(new TestThing(
		 TestThing.BOX_SCALE, TestThing.BALL_DEFAULT));
		 
		affineScl.getTranslation().initActuation(
		 new Vector4d(0, -3, 0, 0));
		getWorld().addSceneNode(affineScl);
		 
		affineScl.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.5, 4));

		/// sphere target, child of affineRot
		AxisAngleSphereGroup sphere =
		 new AxisAngleSphereGroup(new TestThing(
		 TestThing.BOX_DEFAULT, TestThing.BALL_SPHERE));
		 
		AffineGroup sphereAff = new AffineGroup(sphere);
		sphereAff.getTranslation().initActuation(
		 new Vector4d(3, 0, 0, 0));
		affineRot.addNode(sphereAff);

		// setup manipulation controls
		/// manipulation draggers, first button
		InputDragSplitter relDrag = new InputDragSplitter();
		
		ActuationBlocks.buildRelativeDragger(relDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);

		/// manipulation mappers
		InputDragTarget mapper;
		
		//// ex: let event source display define source node
		getView().getView().getViewPlatform().setCapability(
		 Node.ALLOW_LOCAL_TO_VWORLD_READ);
		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affineXlt, null, true);
		relDrag.addEventTarget(mapper);
		
		//// ex: use view object as source node
		mapper = IntuitiveBlocks.buildDrmRotationYMapper(
		 affineRot, getView(), true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmScaleMapper(
		 affineScl, getView(), true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmSphereMapper(
		 sphere, getView(), true);
		relDrag.addEventTarget(mapper);
		 
		// setup over management
		ArrayList pickList, enableList;
		
		pickList = new ArrayList();
		pickList.add(affineXlt);
		pickList.add(sphere);
		pickList.add(affineRot);
		pickList.add(affineScl);
		 
		enableList = new ArrayList();
		enableList.add(affineXlt);
		enableList.add(sphere);
		enableList.add(affineRot);
		enableList.add(affineScl);
		
		IntuitiveBlocks.buildOverEnabler(getView(),
		 getWorld().getSceneRoot(), pickList, enableList);

		// done
		showFrame();
	}
			
	// personal body ============================================

}