import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.media.j3d.*;

import j3dui.utils.blocks.Blocks;

/**
Panel showing a catalog of models as icons.  Intended
for use with other drag-and-drop enabled components. 
<P>
The palette will be loaded from a "catalog" file directory,
with one icon per loadable model file found there. 

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class FancyPalette extends JScrollPane {
	
	// public interface =========================================
	
	/**
	Constructs a FancyPalette from the specified catalog
	directory.
	@param frame Parent application frame.
	*/	
	public FancyPalette(FancyFrame frame) {
		_frame = frame;
		updatePalette();
	}
	
	/**
	Updates this object's contents from the contents of standard
	"catalog" file directories.
	*/	
	public void updatePalette() {
		JPanel palette = buildPalette();
		setViewportView(palette);
		palette.revalidate();
	}
			
	// personal body ============================================

	/** The parent application frame. */
	private FancyFrame _frame;

	/**
	Builds a palette by loading models from standard 'catalog"
	directories as icons.
	@return Panel containing the palette model icons.  Never
	null.
	*/	
	protected JPanel buildPalette()
	{
		JPanel palette = new JPanel(new GridLayout(0, 3, 4, 4));

		addIcons(palette, Blocks.buildResourcePath(
		 "catalog"));
		addIcons(palette, Blocks.buildResourcePath(
		 "../lib/j3dui/utils/objects"));
		
		// done
		return palette;
	}

	/**
	Adds icons to the specified palette representing files in
	the specified directory.
	@param palette Host palette containing the icons.
	@param dirPath Directory path.
	*/	
	protected void addIcons(JPanel palette, String dirPath)
	{
		// get directory files
		File dir = new File(dirPath);
		String[] files = dir.list();
		
		/// build icons, by file		
		ImageIcon iconImage = new ImageIcon(
		 Blocks.buildResourcePath("model.gif"));
		
		for(int fileI=0; fileI<files.length; fileI++) {
			// build icon as label
			final String path = dir.getPath() +
			 java.io.File.separatorChar + files[fileI];

			final JLabel icon = new JLabel(files[fileI],
			 iconImage, JLabel.CENTER);
			icon.setToolTipText(path);
			
			// handle double-click
			icon.addMouseListener(new MouseAdapter() {
				public void mouseClicked(MouseEvent evt) {
					// if double-click, open model
					if(evt.getClickCount()==2) {
						// show border
						icon.setBorder(
						 javax.swing.plaf.BorderUIResource.
						 getEtchedBorderUIResource());
						
						// load new model
						_frame.loadModel(path);
					}
				}
			});
			
			// center it
			icon.setVerticalTextPosition(JLabel.BOTTOM);
			icon.setHorizontalTextPosition(JLabel.CENTER);
		
			palette.add(icon);
		}
	}
	
}