import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.media.j3d.*;

/**
Panel showing a Java 3D scene graph or subgraph as a tree.
<P>
In order for the tree to be generated from the scene graph
either the scene graph must be dead or all the group nodes in
the tree under the specified root node must have their
capability bits set for reading children. 

@author Jon Barrilleaux,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/
public class FancyTree extends JScrollPane {
	
	// public interface =========================================
	
	/**
	Constructs a FancyTree that is empty.
	*/	
	public FancyTree() {}
	
	/**
	Constructs a FancyTree with the specified scene graph.
	@param root Root node of the scene graph.
	*/	
	public FancyTree(Node root) {
		this();
		updateTree(root);
	}
	
	/**
	Updates this object's contents with the specified scene
	graph.
	@param root Root node of the scene graph.
	*/	
	public void updateTree(Node root) {
		JTree tree = new JTree(buildTree(root));
		setViewportView(tree);
		tree.revalidate();
	}
			
	// personal body ============================================

	/**
	Builds a tree by recursively walking the scene graph.
	@param node Root node of the current scene subgraph.
	@return Tree node containing a scene subgraph.  Null if no
	tree.
	*/	
	protected DefaultMutableTreeNode buildTree(Node node)
	{
		if(node == null) return null;
		String name = node.getClass().getName();
		
		DefaultMutableTreeNode treeNode =
		 new DefaultMutableTreeNode(
		 name.substring(name.lastIndexOf('.') + 1)
		);
		
		if(node instanceof Group) {
			Group group = (Group)node;
			final int n = group.numChildren();
			for (int i = 0; i < n; i++)
				treeNode.add(buildTree(group.getChild(i)));
		}
		
		return treeNode;
	}
	
}