import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;

/**
Example demonstrating input drag enabling with modifiers and
splitters.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class InputEnabling extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new InputEnabling();
	}
	
	// public interface =========================================
	
	public InputEnabling() {
		setTitle("Example - InputEnabling");
		
		new HelpDialog(this, "Help - InputEnabling",
		 "Example demonstrating input drag enabling with " +
		 "modifiers and splitters." +
		 "\n" +
		 "\n" +
		 "> Left-button drag in display upper-left to " +
		 "move thing." +
		 "\n" +
		 "> Left-button drag with Shift key to rotate thing " +
		 "about Y-axis." +
		 "\n" +
		 "> Left-button drag with Ctrl key to rotate thing " +
		 "about Z-axis." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup views
		getWorld().addViewNode(
		 BasicBlocks.buildView(getView(),
		 new Vector3d(0, 0, 11), new Vector3d(0, 0, 0)
		));
		
		// setup manipulation targets
		/// create transform chain
		TestThing thing = new TestThing();
		TransformGroup target[] = BasicBlocks.buildTarget(thing);
		getWorld().addSceneNode(target[0]);

		/// connect chain to actuators		
		BasicBlocks.Translator xlt =
		 new BasicBlocks.Translator(target[0]); 
		BasicBlocks.Rotator rotY =
		 new BasicBlocks.Rotator(target[1], 1); 
		BasicBlocks.Rotator rotZ =
		 new BasicBlocks.Rotator(target[1], 2); 
		
		// setup manipulation sensors
		AppDisplay disp = getView().getDisplay();
		Group root = getWorld().getViewRoot();
		
		/// draggers
		InputDragSplitter dragSplt =
		 new InputDragSplitter();
		
		new AwtMouseDragSensor(disp, dragSplt);

		/// modifiers
		InputModifierSplitter modSplt =
		 new InputModifierSplitter();
		
		new AwtKeyboardModifierSensor(disp, modSplt);
		
		// setup manipulation controls
		InputModifierTrigger trigger;
		EnableInputDragFilter enabler;

		/// translation, if None
		enabler = new EnableInputDragFilter(xlt);
		dragSplt.addEventTarget(enabler);
		
		trigger = new InputModifierTrigger(enabler,
		 Input.MODIFIER_NONE, true);
		trigger.initEventTarget(true);
		modSplt.addEventTarget(trigger);
		
		/// rotation Y, if Shift
		enabler = new EnableInputDragFilter(rotY);
		dragSplt.addEventTarget(enabler);
		
		trigger = new InputModifierTrigger(enabler,
		 Input.MODIFIER_SHIFT, true);
		trigger.initEventTarget(false);
		modSplt.addEventTarget(trigger);
		
		/// rotation Z, if Ctrl
		enabler = new EnableInputDragFilter(rotZ);
		dragSplt.addEventTarget(enabler);
		
		trigger = new InputModifierTrigger(enabler,
		 Input.MODIFIER_CTRL, true);
		trigger.initEventTarget(false);
		modSplt.addEventTarget(trigger);
		
		// done
		showFrame();
	}
			
	// personal body ============================================

}