import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;

/**
Example demonstrating the framework input filters and their
features.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class InputFilters extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new InputFilters();
	}
	
	// public interface =========================================
	
	public InputFilters() {
		setTitle("Example - InputFilters");
		
		new HelpDialog(this, "Help - InputFilters",
		 "Example demonstrating the framework input filters " +
		 "and their features." +
		 "\n" +
		 "\n" +
		 "> Left-button drag in display to slide thing." +
		 "\n" +
		 "> Right-button drag left-right to change rate of " +
		 "of thing rotation." +
		 "\n" +
		 "> Middle-button drag around center display tic " +
		 " to rotate thing." +
		 "\n" +
		 "\n" +
		 ">>> Resizing the window will change the relative " +
		 "position of the control origins, which are " +
		 "specified at an absolute display location." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		AppDisplay disp = getView().getDisplay();
		Group root = getWorld().getViewRoot();
		
		// setup views
		getWorld().addViewNode(
		 BasicBlocks.buildView(getView(),
		 new Vector3d(0, 0, 11), new Vector3d(0, 0, 0)
		));
		
		// add control plane to scene
		getWorld().addSceneNode(new TestPlane(
		 TestPlane.PLANE_DEFAULT, 5, 5));

		/// add a control reference mark		
		getWorld().addSceneNode(new TestTic(
		 TestTic.TIC_WHITE, new Vector3d(0, 0, 5)));
		
		// setup manipulation targets
		/// create transform chain
		TestThing thing = new TestThing();
		TransformGroup target[] = BasicBlocks.buildTarget(thing);
		getWorld().addSceneNode(target[0]);

		/// connect chain to actuators		
		BasicBlocks.Translator xlt =
		 new BasicBlocks.Translator(target[0]); 
		BasicBlocks.Rotator rotY =
		 new BasicBlocks.Rotator(target[1], 1); 
		BasicBlocks.Rotator rotZ =
		 new BasicBlocks.Rotator(target[1], 2);
		rotZ.setScale(-1); 

		// setup manipulation controls
		AwtMouseDragSensor dragger;
		InputDragTarget filter;
		
		/// relative clamped translation, first button
		filter = new InputDragFilter(xlt,
		 new ClampInputDragPlugin(new Vector2d(-100, 100)));
		filter = new InputDragFilter(filter,
		 new RelativeInputDragPlugin());
		 
		dragger = new AwtMouseDragSensor(disp, filter);
		dragger.setButtons(Input.BUTTON_FIRST);
		
		/// absolute rate rotation, second button
		filter = new RateInputDragFilter(rotY);
		filter = new InputDragFilter(filter,
		 new AbsoluteInputDragPlugin(new Vector2d(150, -150)));

		dragger = new AwtMouseDragSensor(disp, filter);
		dragger.setButtons(Input.BUTTON_SECOND);

		/// absolute circle rotation, third button
		filter = new InputDragFilter(rotZ,
		 new CircleInputDragPlugin());
		filter = new InputDragFilter(filter,
		 new AbsoluteInputDragPlugin(new Vector2d(150, -150)));

		dragger = new AwtMouseDragSensor(disp, filter);
		dragger.setButtons(Input.BUTTON_THIRD);

		// done
		showFrame();
	}
			
	// personal body ============================================

}