import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;

/**
Example demonstrating many of the framework input
sensors for mouse and keyboard input.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class InputSensors extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new InputSensors();
	}
	
	// public interface =========================================
	
	public InputSensors() {
		setTitle("Example - InputSensors");
		
		new HelpDialog(this, "Help - InputSensors",
		 "Example demonstrating many of the framework input " +
		 "sensors for mouse and keyboard input." +
		 "\n" +
		 "\n" +
		 "> Move mouse in display upper-left to slide thing." +
		 "\n" +
		 "> Left-button drag up-down to rotate thing " +
		 "about X-axis." +
		 "\n" +
		 "> Right-button drag left-right to rotate " +
		 "thing about Y-axis." +
		 "\n" +
		 "> Middle-button (or left-right button) drag " +
		 "left-right to rotate thing about Z-axis." +
		 "\n" +
		 "> Use arrow keys to slide thing." +
		 "\n" +
		 "> Press modifier keys/combinations to " +
		 "change thing colors." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup views
		getWorld().addViewNode(
		 BasicBlocks.buildView(getView(),
		 new Vector3d(0, 0, 11), new Vector3d(0, 0, 0)
		));
		
		// setup manipulation targets
		/// create transform chain
		TestThing thing = new TestThing();
		TransformGroup target[] = BasicBlocks.buildTarget(thing);
		getWorld().addSceneNode(target[0]);

		/// connect chain to actuators		
		BasicBlocks.Translator xlt =
		 new BasicBlocks.Translator(target[0]); 
		BasicBlocks.Rotator rotX =
		 new BasicBlocks.Rotator(target[1], 0); 
		BasicBlocks.Rotator rotY =
		 new BasicBlocks.Rotator(target[1], 1); 
		BasicBlocks.Rotator rotZ =
		 new BasicBlocks.Rotator(target[1], 2); 
		BasicBlocks.Colorer col =
		 new BasicBlocks.Colorer(thing); 

		// setup manipulation controls
		AppDisplay disp = getView().getDisplay();
		Group root = getWorld().getViewRoot();
		
		/// move translation
		new AwtMouseMoveSensor(disp, xlt);
		 
		/// arrow translation
		new AwtKeyboardDragSensor(disp, xlt);

		/// drag rotation
		AwtMouseDragSensor dragger;
		
		//// rotate X		
		dragger = new AwtMouseDragSensor(disp, rotX);		
		dragger.setButtons(Input.BUTTON_FIRST);

		//// rotate Y		
		dragger = new AwtMouseDragSensor(disp, rotY);
		dragger.setButtons(Input.BUTTON_SECOND);
		
		//// rotate Z		
		dragger = new AwtMouseDragSensor(disp, rotZ);
		dragger.setButtons(Input.BUTTON_THIRD);

		/// modifier color
		AwtKeyboardModifierSensor modifier;
		modifier = new AwtKeyboardModifierSensor(disp, col);
		modifier.setModifiers(Input.MODIFIER_ALL_ESC);

		// done
		showFrame();
	}
			
	// personal body ============================================

}