import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;
import j3dui.feedback.elements.*;
import j3dui.visualize.*;
import j3dui.visualize.change.*;

/**
Example demonstrating the use of multiple displays and some
of the problems.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class MultiDisplaying extends DoubleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new MultiDisplaying();
	}
	
	// public interface =========================================
	
	public MultiDisplaying() {
		setTitle("Example - MultiDisplaying");
		
		new HelpDialog(this, "Help - MultiDisplaying",
		 "Example demonstrating the use of multiple displays " +
		 "and some of the problems." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn a view." +
		 "\n" +
		 "> Left-button drag or arrow keys to turn " +
		 "side things." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "change display FOV." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Red thing tooltip only seen in and remains " +
		 "valid for the last view manipulated." +
		 "\n" +
		 ">>> Java 3D 1.1 switch bug was fixed in 1.2, so gray " +
		 "thing in second view no longer jumps upon state change " +
		 "(e.g. mouse-over)." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getViewLeft(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0), 10,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getViewRight(),
		 new Vector3d(-Math.PI/8, Math.PI/8, 0), 10,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);

		// build sensors
		ViewChangeSplitter viewSpltr = new ViewChangeSplitter();
		
		new ViewExternalChangeSensor(viewSpltr, getViewLeft());
		new ViewExternalChangeSensor(viewSpltr, getViewRight());
		
		new ViewInternalChangeSensor(viewSpltr, getViewLeft());
		new ViewInternalChangeSensor(viewSpltr, getViewRight());

		// build world: normal center thing
		AffineGroup affine;
		Node thing;

		thing = new TestThing(TestThing.BOX_DISABLE_0,
		 TestThing.BALL_CANCEL);
		getWorld().addSceneNode(thing);

		// build world: side rotating things
		/// side thing rotator		
		AffineGroup affineY = new AffineGroup();
		getWorld().addSceneNode(affineY);
		
		/// normal multishape side		
		MultiShape multi0 = FeedbackBlocks.buildMultiThing();
		affine = new AffineGroup(multi0);
		affine.getTranslation().initActuation(
		 new Vector4d(-3, 0, 0, 0));
		affineY.addNode(affine);
		 
		/// perfect overlay side
		double overlay;
		double size = .025;
				
		//// perfect callout
		overlay = 0.2;
		
		thing = FeedbackBlocks.buildMultiTooltip(false, 3, .75,
		 Blocks.buildResourcePath("feedback/CalloutSlide.gif"));
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(0, .5, 0, 0));
		
		PerfectOverlayGroup perfect0 =
		 new PerfectOverlayGroup(affine,
		 Visualize.DO_ALL, overlay, size);
		viewSpltr.addEventTarget(perfect0);
		 
		affine = new AffineGroup(perfect0);
		affine.getTranslation().initActuation(
		 new Vector4d(3, 0, 0, 0));
		affineY.addNode(affine);
				
		//// normal thing
		thing = new TestThing(TestThing.BOX_NORMAL_1,
		 TestThing.BALL_DEFAULT);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(3, -1, 0, 0));
		affineY.addNode(affine);
		
		// build manipulation controls
		InputDragTarget mapper;
		InputDragSplitter dragSpltr = new InputDragSplitter();
		
		/// side thing Y rotation, first + none
		mapper = ActuationBlocks.buildDirectMapper(
		 affineY.getAxisAngle(), new Vector2d(.01, -.01),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		affineY.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		dragSpltr.addEventTarget(mapper);
 
		ActuationBlocks.buildRelativeDragger(dragSpltr,
		 getViewLeft(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		ActuationBlocks.buildRelativeDragger(dragSpltr,
		 getViewRight(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);

		/// view fov, first + control
		VisualizeBlocks.buildViewFover(getViewLeft(),
		 new Vector2d(Math.PI/6, Math.PI/2.5), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);
		VisualizeBlocks.buildViewFover(getViewRight(),
		 new Vector2d(Math.PI/6, Math.PI/2.5), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);

		// setup target feedback
		FeedbackGroupManager groupManager =
		 FeedbackBlocks.buildGroupManager(
		 getViewLeft().getDisplay(), getWorld().getViewRoot());

		FeedbackBlocks.buildTargetFeedbackSupport(
		 multi0, groupManager, dragSpltr,
		 getViewLeft().getDisplay(), getWorld().getSceneRoot());

		FeedbackBlocks.buildTargetFeedbackSupport(
		 multi0, groupManager, dragSpltr,
		 getViewRight().getDisplay(), getWorld().getSceneRoot());
		
		groupManager.initTargets(
		 null, Feedback.STATUS_NORMAL,
		 Feedback.SELECT_NORMAL, Feedback.ACTION_NORMAL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}