import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;

/**
Example demonstrating multishape feedback, sensors, and
filters.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class MultiShaping extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new MultiShaping();
	}
	
	// public interface =========================================
	
	public MultiShaping() {
		setTitle("Example - MultiShaping");
		
		new HelpDialog(this, "Help - MultiShaping",
		 "Example demonstrating multishape feedback, " +
		 "sensors, and filters." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag things to slide them." +
		 "\n" +
		 "> Use arrow keys with mouse over thing to " +
		 "manipulate it." +
		 "\n" +
		 "> Escape key to cancel selection and manipulation." +
		 "\n" +
		 "> Single, double, triple click things." +
		 "\n" +
		 "\n" +
		 ">>> Things change shape (color) according to " +
		 "status, selection, and action state." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup multishape targets
		/// target 0
		MultiShape multi0 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine0 = new AffineGroup(multi0);
		 
		affine0.getTranslation().initActuation(
		 new Vector4d(0, 2, 0, 0));
		getWorld().addSceneNode(affine0);

		/// target 1
		MultiShape multi1 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine1 = new AffineGroup(multi1);
		 
		affine1.getTranslation().initActuation(
		 new Vector4d(-2, -2, 0, 0));
		getWorld().addSceneNode(affine1);

		/// target 2
		MultiShape multi2 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine2 = new AffineGroup(multi2);
		 
		affine2.getTranslation().initActuation(
		 new Vector4d(2, -2, 0, 0));
		getWorld().addSceneNode(affine2);
		
		// setup manipulation controls
		getView().getView().getViewPlatform().setCapability(
		 Node.ALLOW_LOCAL_TO_VWORLD_READ);
		 
		/// manipulation draggers, first button
		InputDragSplitter relDrag = new InputDragSplitter();
		
		ActuationBlocks.buildRelativeDragger(relDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// manipulation mappers
		InputDragTarget mapper;

		//// translation		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine0, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine1, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine2, null, true);
		relDrag.addEventTarget(mapper);
		// setup manipulation feedback
		/// build group manager (no multi-select)
		FeedbackGroupManager groupManager = FeedbackBlocks.
		 buildGroupManager(getView().getDisplay(),
		 getWorld().getViewRoot());
		groupManager.getSelectManager().setMultiSelect(
		 Input.MODIFIER_IGNORE);

		/// add targets
		StatusTrigger trigger;
		BranchGroup pickRoot = getWorld().getSceneRoot();
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi0, groupManager, relDrag, getView().getDisplay(), pickRoot);
		affine0.setEventTargetEnable(trigger);
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi1, groupManager, relDrag, getView().getDisplay(), pickRoot);
		affine1.setEventTargetEnable(trigger);
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi2, groupManager, relDrag, getView().getDisplay(), pickRoot);
		affine2.setEventTargetEnable(trigger);

		/// initialize feedback (last)
		groupManager.initTargets(null, Feedback.STATUS_DISABLE,
		 Feedback.SELECT_NORMAL, Feedback.ACTION_NORMAL);
		 
		// setup over management
		ArrayList pickList, enableList;
		
		pickList = new ArrayList();
		pickList.add(multi0);
		pickList.add(multi1);
		pickList.add(multi2);
		 
		enableList = new ArrayList();
		enableList.add(affine0);
		enableList.add(affine1);
		enableList.add(affine2);
		
		IntuitiveBlocks.buildOverEnabler(getView(),
		 getWorld().getSceneRoot(), pickList, enableList);
		 
		// done
		showFrame();
	}
			
	// personal body ============================================

}