import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;

/**
Example demonstrating picking and over-target control
enabling.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class OverEnabling extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new OverEnabling();
	}
	
	// public interface =========================================
	
	public OverEnabling() {
		setTitle("Example - OverEnabling");
		
		new HelpDialog(this, "Help - OverEnabling",
		 "Example demonstrating discrete picking and " +
		 "over-target control enabling." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag or red thing to slide it." +
		 "\n" +
		 "> Left-button drag green thing to rotate it." +
		 "\n" +
		 "> Left-button drag blue thing to scale it." +
		 "\n" +
		 "> Left-button drag magenta thing to roll it." +
		 "\n" +
		 "> Use arrow keys with mouse over thing to " +
		 "manipulate it." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup manipulation targets
		/// translation target
		AffineGroup affineXlt = new AffineGroup(new TestThing(
		 TestThing.BOX_TRANSLATION, TestThing.BALL_DEFAULT));
		getWorld().addSceneNode(BasicBlocks.buildTarget(
		 affineXlt, new Vector3d(0, 3, 0))[0]);
		
		/// rotation target
		AffineGroup affineRot = new AffineGroup(new TestThing(
		 TestThing.BOX_ROTATION, TestThing.BALL_DEFAULT));
		getWorld().addSceneNode(BasicBlocks.buildTarget(
		 affineRot, new Vector3d(0, 0, 0))[0]);
		
		affineRot.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		
		/// scale target
		AffineGroup affineScl = new AffineGroup(new TestThing(
		 TestThing.BOX_SCALE, TestThing.BALL_DEFAULT));
		getWorld().addSceneNode(BasicBlocks.buildTarget(
		 affineScl, new Vector3d(0, -3, 0))[0]);
		 
		affineScl.getScale().getPlugin().setSourceOffset(
		 new Vector4d(1, 1, 1, 0));
		affineScl.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.5, 4));

		/// sphere target, child of affineRot
		RotationSphereGroup sphere =
		 new RotationSphereGroup(new TestThing(
		 TestThing.BOX_DEFAULT, TestThing.BALL_SPHERE));
		affineRot.addNode(BasicBlocks.buildTarget(
		 sphere, new Vector3d(3, 0, 0))[0]);

		// setup manipulation controls
		InputDragMapper mapper;
		
		/// translation, first button
		mapper = ActuationBlocks.buildDirectMapper(
		 affineXlt.getTranslation(), new Vector2d(.025, .025),
		 Mapper.DIM_X, Mapper.DIM_Y, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// rotation, first button
		mapper = ActuationBlocks.buildDirectMapper(
		 affineRot.getAxisAngle(), new Vector2d(.025, .025),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// scale, first button
		mapper = ActuationBlocks.buildDirectMapper(
		 affineScl.getScale(), new Vector2d(.025, .025),
		 Mapper.DIM_X, Mapper.DIM_Y, true);
		 
		ActuationBlocks.buildRelativeDragger(mapper,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// sphere, first button
		InputDragSplitter splitter = new InputDragSplitter();
		
		mapper = ActuationBlocks.buildDirectMapper(
		 sphere.getRotationX(), new Vector2d(.025, -.025),
		 Mapper.DIM_NONE, Mapper.DIM_W, true);
		splitter.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 sphere.getRotationY(), new Vector2d(.025, .025),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		splitter.addEventTarget(mapper);
		 
		ActuationBlocks.buildRelativeDragger(splitter,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		 
		// setup over management
		ArrayList pickList, enableList;
		
		pickList = new ArrayList();
		pickList.add(affineXlt);
		pickList.add(sphere);
		pickList.add(affineRot);
		pickList.add(affineScl);
		 
		enableList = new ArrayList();
		enableList.add(affineXlt);
		enableList.add(sphere);
		enableList.add(affineRot);
		enableList.add(affineScl);
		
		IntuitiveBlocks.buildOverEnabler(getView(),
		 getWorld().getSceneRoot(), pickList, enableList);

		// done
		showFrame();
	}
			
	// personal body ============================================

}