import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;
import j3dui.feedback.elements.*;
import j3dui.visualize.*;
import j3dui.visualize.change.*;

/**
Example demonstrating the perfect overlay visualization
technique.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class PerfectOverlaying extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new PerfectOverlaying();
	}
	
	// public interface =========================================
	
	public PerfectOverlaying() {
		setTitle("Example - PerfectOverlaying");
		
		new HelpDialog(this, "Help - PerfectOverlaying",
		 "Example demonstrating the perfect overlay " +
		 "visualization technique." +
		 "\n" +
		 "\n" +
		 "> Resize display window." +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag or arrow keys to turn " +
		 "side things." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "change display FOV." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Red thing decorations remain perfectly " +
		 "oriented, sized, and overlaid regardless of how " +
		 "thing, view, or display is manipulated." +
		 "\n" +
		 ">>> Gray thing decoration is a billboard." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// setup orbitable view, first + shift
		getView().setViewFov(Math.PI/2);
		
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(0, 0, 0), 5,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);

		// build sensors
		ViewChangeSplitter viewSpltr = new ViewChangeSplitter();
		
		new ViewExternalChangeSensor(viewSpltr, getView());
		new ViewInternalChangeSensor(viewSpltr, getView());

		// build world: normal center thing
		AffineGroup affine;
		Node thing;

		thing = new TestThing(TestThing.BOX_DISABLE_0,
		 TestThing.BALL_CANCEL);
		getWorld().addSceneNode(thing);

		// build world: side rotating things
		/// side thing rotator with ordered group		
		AffineGroup affineY = new AffineGroup();
		getWorld().addSceneNode(affineY);
		
// Java 3D 1.1.2 transparency overlap bug fixed so no
// longer need ordered node (and don't want it because it
// now forces the wrong overlap order).		
//		OrderedGroup ordered = new OrderedGroup();
Group ordered = new Group();
		affineY.addNode(ordered);

		/// normal side
		//// billboard callout		
		thing = FeedbackBlocks.buildMultiTooltip(false, 3, .75,
		 Blocks.buildResourcePath("feedback/CalloutSlide.gif"));
		
		TransformGroup xform = new TransformGroup();
		xform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		xform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		xform.addChild(thing);
		
		Billboard bboard = new Billboard(xform,
		 Billboard.ROTATE_ABOUT_POINT, new Point3f(0f, 0f, 0f));
		bboard.setSchedulingBounds(new BoundingSphere(
		 new Point3d(0,0,0), Double.POSITIVE_INFINITY));
		xform.addChild(bboard);
		
		affine = new AffineGroup(xform);
		affine.getTranslation().initActuation(
		 new Vector4d(-3, 0, 0, 0));
		ordered.addChild(affine);

		//// normal thing		
		thing = new TestThing(TestThing.BOX_NORMAL_0,
		 TestThing.BALL_OVER);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(-3, -1.5, 0, 0));
		ordered.addChild(affine);
		 
		/// perfect overlay side
		double overlay;
		double size = .025;
				
		//// perfect callout
		overlay = 0.175;
		
		thing = FeedbackBlocks.buildMultiTooltip(false, 3, .75,
		 Blocks.buildResourcePath("feedback/CalloutSlide.gif"));
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(0, 1.2, 0, 0));
		
		PerfectOverlayGroup perfect0 =
		 new PerfectOverlayGroup(affine,
		 Visualize.DO_ALL, overlay, size);
		viewSpltr.addEventTarget(perfect0);
		 
		affine = new AffineGroup(perfect0);
		affine.getTranslation().initActuation(
		 new Vector4d(3, -.5, 0, 0));
		ordered.addChild(affine);

		//// near perfect (over+size) handle
		overlay = 0.2;
		
		thing = new TextureCylinder(false, null, 0, 
		 Blocks.buildResourcePath("feedback/OverRotate.gif"),
		 Elements.SIDE_ALL, 1, .5, .75);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(0, .05, 0, 0));
		
		PerfectOverlayGroup perfect1 =
		 new PerfectOverlayGroup(affine,
		 Visualize.DO_OVER_SIZE, overlay, size);
		viewSpltr.addEventTarget(perfect1);
		 
		affine = new AffineGroup(perfect1);
		affine.getTranslation().initActuation(
		 new Vector4d(3, -.5, 0, 0));
		ordered.addChild(affine);
				
		//// normal thing
		thing = new TestThing(TestThing.BOX_NORMAL_1,
		 TestThing.BALL_OVER);
		affine = new AffineGroup(thing);
		affine.getTranslation().initActuation(
		 new Vector4d(3, -1.5, 0, 0));
		ordered.addChild(affine);
		
		// build manipulation controls
		InputDragTarget mapper;
		InputDragSplitter dragSpltr = new InputDragSplitter();
		
		/// side thing Y rotation, first + none
		mapper = ActuationBlocks.buildDirectMapper(
		 affineY.getAxisAngle(), new Vector2d(.01, -.01),
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		affineY.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		dragSpltr.addEventTarget(mapper);
 
		ActuationBlocks.buildRelativeDragger(dragSpltr,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);

		/// view fov, first + control
		VisualizeBlocks.buildViewFover(getView(),
		 new Vector2d(Math.PI/4, Math.PI/1.5), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}