import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.inputs.sensors.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;
import j3dui.feedback.elements.*;
import j3dui.visualize.*;
import j3dui.visualize.change.*;

/**
Example demonstrating second person manipulation
with perfectly overlaid target decorations.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class SecondPerson extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new SecondPerson();
	}
	
	// public interface =========================================
	
	public SecondPerson() {
		setTitle("Example - SecondPerson");
		
		new HelpDialog(this, "Help - SecondPerson",
		 "Example demonstrating second person manipulation " +
		 "with perfectly overlaid target decorations." +
		 "\n" +
		 "\n" +
		 "> Resize display window." +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Single/double/triple click things to enable " +
		 "sliding/rotating/lifting them." +
		 "\n" +
		 "> Single/double/triple-ctrl click things to " +
		 "group enable them." +
		 "\n" +
		 "> Left-button drag or use arrow keys to manipulate " +
		 "selected thing or group." +
		 "\n" +
		 "> Escape key to cancel selection and manipulation." +
		 "\n" +
		 "> Slide lift control to lift things selected for " +
		 "lifting." +
		 "\n" +
		 "\n" +
		 ">>> Mouse-over, select, and/or drag for visual " +
		 "and audio feedback." +
		 "\n" +
		 ">>> Display decorations remain perfectly " +
		 "overlaid  regardless of how things, view, or " +
		 "display is manipulated." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// setup orbitable view, first+shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/6, -Math.PI/8, 0), 10,
		 Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);

		// build view change sensor
		ViewChangeSplitter viewSpltr = new ViewChangeSplitter();
		
		new ViewExternalChangeSensor(viewSpltr, getView());
		new ViewInternalChangeSensor(viewSpltr, getView());

		// build room with floor
		SecondBlocks.Room room = new SecondBlocks.Room(
		 new Vector3d(8, 3, 8));
		getWorld().addSceneNode(room);
		
		// build target draggers
		/// absolute draggers (for WRM slide)
		InputDragSplitter absDrag = new InputDragSplitter();
		
		ActuationBlocks.buildAbsoluteDragger(absDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// relative draggers (for direct rotate & lift)
		InputDragSplitter relDrag = new InputDragSplitter();
		
		ActuationBlocks.buildRelativeDragger(relDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		// build feedback manager
		FeedbackGroupManager groupMgr =
		 FeedbackBlocks.buildGroupManager(
		 getView().getDisplay(), getWorld().getViewRoot());

		/// use room for deselect
		FeedbackBlocks.buildTargetFeedbackSupport(room,
		 groupMgr, relDrag, getView().getDisplay(),
		 getWorld().getSceneRoot());
				
		// build targets
		Node thing;
		SecondBlocks.FloorTarget target;
		
		/// lamp
		thing = ModelLoader.loadModelFromName(
		 Blocks.buildResourcePath("catalog/lamp.wrl"));
		
		target = new SecondBlocks.FloorTarget(
		 thing, room, viewSpltr, absDrag, relDrag, groupMgr,
		 getWorld(), getView());
		target.getSlide().initActuation(
		 new Vector4d(-.45, 0, -.3, 0));
		target.getLift().initActuation(
		 new Vector4d(0, 1.65, 0, 0));
		getWorld().addSceneNode(target);

		/// desk
		thing = ModelLoader.loadModelFromName(
		 Blocks.buildResourcePath("catalog/desk.wrl"));
		
		target = new SecondBlocks.FloorTarget(
		 thing, room, viewSpltr, absDrag, relDrag, groupMgr,
		 getWorld(), getView());
		getWorld().addSceneNode(target);

		/// drawers
		thing = ModelLoader.loadModelFromName(
		 Blocks.buildResourcePath("catalog/drawers.wrl"));
		
		target = new SecondBlocks.FloorTarget(
		 thing, room, viewSpltr, absDrag, relDrag, groupMgr,
		 getWorld(), getView());
		target.getSlide().initActuation(
		 new Vector4d(1.2, 0, .05, 0));
		target.getLift().initActuation(
		 new Vector4d(0, .55, 0, 0));
		getWorld().addSceneNode(target);
		
		// initialize feedback (last)
		groupMgr.initTargets(null, Feedback.STATUS_DISABLE,
		 Feedback.SELECT_NORMAL, Feedback.ACTION_NORMAL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}