import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;

/**
An example of a 'simple' application with dual displays that
uses the framework application and debug utilities.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class SimpleApp extends Frame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		Debug.println("hello.world", "Hello world!");
		Debug.println("hello.sun", "Hello sun!");
		
		new SimpleApp();
	}
	
	// public interface =========================================
	
	public SimpleApp() {
		super("Example - SimpleApp");
		
		new HelpDialog(this, "Help - SimpleApp",
		 "An example of a 'simple' application with dual " +
		 "displays that uses the " +
		 "framework application and debug utilities." +
		 "\n" +
		 "\n" +
		 ">>> No user interaction." +
		 "\n" +
		 ">>> If debug tags were set debug messages should have " +
		 "printed to the command window." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// kill the window on close
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent winEvent) {
				System.exit(0);
			}
		});
		
		// build the UI spaces
		AppWorld world = new AppWorld();
		AppView viewL = new AppView();
		AppView viewR = new AppView();

		// add the displays to the window
		setLayout(new GridLayout(1,2,2,2));
		add(viewL.getDisplay());
		add(viewR.getDisplay());

		// build and add the scene objects
		world.addSceneNode(new TestThing());

		// build and add the view objects
		world.addViewNode(
		 BasicBlocks.buildView(viewL,
		 new Vector3d(-2.5, 0, 3),
		 new Vector3d(0, -Math.PI/5, 0)
		));
		 
		world.addViewNode(
		 BasicBlocks.buildView(viewR,
		 new Vector3d(2.5, 0, 3),
		 new Vector3d(0, Math.PI/5, 0)
		));

		// make the scene graph live
		world.setLive(true);

		// size and show the window
		setSize(400, 200);
		show();
	}
}