import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;

/**
Example demonstrating multi-shape target decorations for visual
and audio feedback (tooltips, handles, sound effects).
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class TargetDecorating extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new TargetDecorating();
	}
	
	// public interface =========================================
	
	public TargetDecorating() {
		setTitle("Example - TargetDecorating");
		
		new HelpDialog(this, "Help - TargetDecorating",
		 "Example demonstrating multi-shape target " +
		 "decorations for visual and audio feedback " +
		 "(tooltips, handles, sound effects)." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Single click things to enable sliding them." +
		 "\n" +
		 "> Double click things to enable rotating them." +
		 "\n" +
		 "> Triple click things to enable lifting them." +
		 "\n" +
		 "> Single/double/triple-ctrl click things to " +
		 "group enable them." +
		 "\n" +
		 "> Left-button drag or use arrow keys to manipulate " +
		 "selected thing or group." +
		 "\n" +
		 "> Escape key to cancel selection and manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Mouse-over selected thing and and pause for " +
		 "tooltip." +
		 "\n" +
		 ">>> Mouse-over, select, and drag things to hear " +
		 "sound effects." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		AffineGroup affine;
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/6, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);

		// build floor plane
		TestPlane floor = new TestPlane(
		 TestPlane.PLANE_DEFAULT, 8, 8);
		 
		affine = new AffineGroup(floor);
		affine.getTranslation().initActuation(
		 new Vector4d(0, 0, 0, 0));
		affine.getAxisAngle().initActuation(
		 new Vector4d(1, 0, 0, -Math.PI/2.0));
		getWorld().addSceneNode(affine);
		
		// setup multishape targets
		MultiShape thing;
		
		/// target 0
		thing = FeedbackBlocks.buildMultiThing();
		AffineGroup slide0 = new AffineGroup();
		AffineGroup rotate0 = new AffineGroup();
		AffineGroup lift0 = new AffineGroup();
		MultiShape target0 =
		 FeedbackBlocks.buildMultiFloorTarget(
		 thing, slide0, rotate0, lift0, getView(), 1);
		getWorld().addSceneNode(target0);
		
		slide0.getTranslation().initActuation(
		 new Vector4d(0, 1, 2.5, 0));

		/// target 1
		thing = FeedbackBlocks.buildMultiThing();
		AffineGroup slide1 = new AffineGroup();
		AffineGroup rotate1 = new AffineGroup();
		AffineGroup lift1 = new AffineGroup();
		MultiShape target1 =
		 FeedbackBlocks.buildMultiFloorTarget(
		 thing, slide1, rotate1, lift1, getView(), 1);
		getWorld().addSceneNode(target1);
		
		slide1.getTranslation().initActuation(
		 new Vector4d(-2.5, 1, -2.5, 0));

		/// target 2
		thing = FeedbackBlocks.buildMultiThing();
		AffineGroup slide2 = new AffineGroup();
		AffineGroup rotate2 = new AffineGroup();
		AffineGroup lift2 = new AffineGroup();
		MultiShape target2 =
		 FeedbackBlocks.buildMultiFloorTarget(
		 thing, slide2, rotate2, lift2, getView(), 1);
		getWorld().addSceneNode(target2);
		
		slide2.getTranslation().initActuation(
		 new Vector4d(2.5, 1, -2.5, 0));
	
		// setup manipulation controls
		/// absolute draggers (for WRM slide)
		InputDragSplitter absDrag = new InputDragSplitter();
		ActuationBlocks.buildAbsoluteDragger(absDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// relative draggers (for direct rotate)
		InputDragSplitter relDrag = new InputDragSplitter();
		ActuationBlocks.buildRelativeDragger(relDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// manipulation mappers
		InputDragTarget mapper;
		Vector2d rotateScale = new Vector2d(.025, .025);
		Vector2d liftScale = new Vector2d(.025, .025);
		
		//// WRM slide		
		mapper = IntuitiveBlocks.buildStickyWrmTranslationMapper(
		 floor, slide0, getWorld().getSceneRoot(), true, true);
		absDrag.addEventTarget(mapper);

		mapper = IntuitiveBlocks.buildStickyWrmTranslationMapper(
		 floor, slide1, getWorld().getSceneRoot(), true, true);
		absDrag.addEventTarget(mapper);

		mapper = IntuitiveBlocks.buildStickyWrmTranslationMapper(
		 floor, slide2, getWorld().getSceneRoot(), true, true);
		absDrag.addEventTarget(mapper);
		
		//// direct rotate
		mapper = ActuationBlocks.buildDirectMapper(
		 rotate0.getAxisAngle(), rotateScale,
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		rotate0.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		relDrag.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 rotate1.getAxisAngle(), rotateScale,
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		rotate1.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		relDrag.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 rotate2.getAxisAngle(), rotateScale,
		 Mapper.DIM_W, Mapper.DIM_NONE, true);
		rotate2.getAxisAngle().getPlugin().setSourceOffset(
		 new Vector4d(0, 1, 0, 0));
		relDrag.addEventTarget(mapper);

		//// direct lift
		mapper = ActuationBlocks.buildDirectMapper(
		 lift0.getTranslation(), liftScale,
		 Mapper.DIM_NONE, Mapper.DIM_Y, true);
		relDrag.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 lift1.getTranslation(), liftScale,
		 Mapper.DIM_NONE, Mapper.DIM_Y, true);
		relDrag.addEventTarget(mapper);

		mapper = ActuationBlocks.buildDirectMapper(
		 lift2.getTranslation(), liftScale,
		 Mapper.DIM_NONE, Mapper.DIM_Y, true);
		relDrag.addEventTarget(mapper);

		// setup target feedback and enable
		/// build group manager
		FeedbackGroupManager groupManager =
		 FeedbackBlocks.buildGroupManager(
		 getView().getDisplay(), getWorld().getViewRoot());

		/// add targets
		StatusTrigger trigger;
		BranchGroup pickRoot = getWorld().getSceneRoot();
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 target0, groupManager, relDrag, getView().getDisplay(),
		 pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 target0, trigger, slide0.getTranslation(),
		 rotate0.getAxisAngle(), lift0.getTranslation());
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 target1, groupManager, relDrag, getView().getDisplay(),
		 pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 target1, trigger, slide1.getTranslation(),
		 rotate1.getAxisAngle(), lift1.getTranslation());
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 target2, groupManager, relDrag, getView().getDisplay(),
		 pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 target2, trigger, slide2.getTranslation(),
		 rotate2.getAxisAngle(), lift2.getTranslation());
		
		/// initialize feedback (last)
		groupManager.initTargets(null, Feedback.STATUS_DISABLE,
		 Feedback.SELECT_NORMAL, Feedback.ACTION_NORMAL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}