import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.*;

/**
Example demonstrating target and group selection management.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class TargetSelecting extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new TargetSelecting();
	}
	
	// public interface =========================================
	
	public TargetSelecting() {
		setTitle("Example - TargetSelecting");
		
		new HelpDialog(this, "Help - TargetSelecting",
		 "Example demonstrating target and group selection " +
		 "management." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Single click things to enable sliding them." +
		 "\n" +
		 "> Double click things to enable rotating them." +
		 "\n" +
		 "> Triple click things to enable scaling them." +
		 "\n" +
		 "> Single/double/triple-ctrl click things to " +
		 "group enable them." +
		 "\n" +
		 "> Left-button drag or use arrow keys to manipulate " +
		 "selected thing or group." +
		 "\n" +
		 "> Escape key to cancel selection and manipulation." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable views, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, -Math.PI/8, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup multishape targets
		/// target 0
		MultiShape multi0 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine0 = new AffineGroup(multi0);
		 
		affine0.getTranslation().initActuation(
		 new Vector4d(0, 2, 0, 0));
		getWorld().addSceneNode(affine0);
		 
		affine0.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.5, 4));

		/// target 1
		MultiShape multi1 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine1 = new AffineGroup(multi1);
		 
		affine1.getTranslation().initActuation(
		 new Vector4d(-2, -2, 0, 0));
		getWorld().addSceneNode(affine1);
		 
		affine1.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.5, 4));

		/// target 2
		MultiShape multi2 = FeedbackBlocks.buildMultiThing();
		AffineGroup affine2 = new AffineGroup(multi2);
		 
		affine2.getTranslation().initActuation(
		 new Vector4d(2, -2, 0, 0));
		getWorld().addSceneNode(affine2);
		 
		affine2.getScale().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(.5, 4));
		
		// setup manipulation controls
		getView().getView().getViewPlatform().setCapability(
		 Node.ALLOW_LOCAL_TO_VWORLD_READ);
		 
		/// manipulation draggers, first button
		InputDragSplitter relDrag = new InputDragSplitter();
		
		ActuationBlocks.buildRelativeDragger(relDrag,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// manipulation mappers
		InputDragTarget mapper;

		//// translation		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine0, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine1, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmTranslationMapper(
		 affine2, null, true);
		relDrag.addEventTarget(mapper);
		
		//// rotation
		mapper = IntuitiveBlocks.buildDrmRotationYMapper(
		 affine0, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmRotationYMapper(
		 affine1, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmRotationYMapper(
		 affine2, null, true);
		relDrag.addEventTarget(mapper);
		
		//// scale
		mapper = IntuitiveBlocks.buildDrmScaleMapper(
		 affine0, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmScaleMapper(
		 affine1, null, true);
		relDrag.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildDrmScaleMapper(
		 affine2, null, true);
		relDrag.addEventTarget(mapper);

		// setup target feedback and enable
		/// build group manager
		FeedbackGroupManager groupManager =
		 FeedbackBlocks.buildGroupManager(
		 getView().getDisplay(), getWorld().getViewRoot());

		/// add targets
		StatusTrigger trigger;
		BranchGroup pickRoot = getWorld().getSceneRoot();
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi0, groupManager, relDrag, getView().getDisplay(), pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 multi0, trigger, affine0.getTranslation(),
		 affine0.getAxisAngle(), affine0.getScale());
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi1, groupManager, relDrag, getView().getDisplay(), pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 multi1, trigger, affine1.getTranslation(),
		 affine1.getAxisAngle(), affine1.getScale());
		
		trigger = FeedbackBlocks.buildTargetFeedbackSupport(
		 multi2, groupManager, relDrag, getView().getDisplay(), pickRoot);
		FeedbackBlocks.buildTargetSelectEnablers(
		 multi2, trigger, affine2.getTranslation(),
		 affine2.getAxisAngle(), affine2.getScale());
		
		/// initialize feedback (last)
		groupManager.initTargets(null, Feedback.STATUS_DISABLE,
		 Feedback.SELECT_NORMAL, Feedback.ACTION_NORMAL);

		// done
		showFrame();
	}
			
	// personal body ============================================

}