import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.blocks.*;
import j3dui.utils.objects.*;
import j3dui.control.*;
import j3dui.control.mappers.*;
import j3dui.control.actuators.groups.*;
import j3dui.feedback.elements.*;
import j3dui.navigate.*;

/**
Example demonstrating textured and shaded shapes.
<P>
The earth rectangle is solid and textured.  The cone is shaded
and textured.  The thing at the at the origin is gray.  The
view headlight is a narrow red spotlight.  Note its affect of
individual facets as the view orbits and moves in and out.
<P>
This example was developed after the book was printed and, as
such, is not described in the book.

@author Jon Barrilleax,
copyright (c) 1999-2000 Jon Barrilleaux,
All Rights Reserved.
*/

public class TextureShading extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new TextureShading();
	}
	
	// public interface =========================================
	
	public TextureShading() {
		setTitle("Example - TextureShading");
		
		new HelpDialog(this, "Help - TextureShading",
		 "Example demonstrating textured and shaded shapes." +
		 "\n" +
		 "\n" +
		 "> Left-button drag or arrow keys to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button-ctrl drag or arrow keys-ctrl to " +
		 "change LFO-Z." +
		 "\n" +
		 "> Left-button-shift-ctrl drag or arrow " +
		 "keys-shift-ctrl to slide LAP in X-Z using DRM." +
		 "\n" +
		 "\n" +
		 ">>> The earth rectangle is solid and textured.  The " +
		 "cone is shaded and textured.  The thing at the " +
		 "at the origin is gray.  The view headlight is a narrow " +
		 "red spotlight.  Note its affect of individual facets " +
		 "as the view orbits and moves in and out." +
		 "\n" +
		 "\n" +
		 "This example is not described in the book because " +
		 "it was developed after the book was printed." +
		 "\n" +
		 "\n" +
		 "copyright (c) 1999-2000 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);

		// build orbit camera and controls
		OrbitCamera camera = new OrbitCamera(getView());
		new OrbitCameraControl(getView().getDisplay(), camera);
		
		getWorld().addViewNode(camera);

		/// start looking from the south west
		camera.getOrbitActuator().initActuation(
		 new Vector4d(-Math.PI/6, -Math.PI/8, 0, 0));
		 
		// build red spotlight as the view headlight
		SpotLight light = new SpotLight();
		light.setSpreadAngle((float)Math.PI/16);
		light.setColor(new Color3f(1f, 0f, 0f));
		
		getView().addNode(new TestLight(light));

		// build gray thing at origin
		getWorld().addSceneNode(new TestThing(
		 new Color3f(.5f, .5f, .5f),
		 new Color3f(.5f, .5f, .5f)));

		// build textured ground
		/// build textured rect
		TextureRect rect = new TextureRect(
		 Elements.SIDE_ALL, 10.0, 10.0);
		rect.setTexture("earth.jpg");

		/// flip it up		
		AffineGroup ground = new AffineGroup(rect);
		ground.getAxisAngle().initActuation(
		 new Vector4d(1, 0, 0, -Math.PI/2.0));
		
		getWorld().addSceneNode(ground);

		// build textured and shaded cone
		TextureCone cone = new TextureCone(false,
		 new Color3f(), .33, "sky.jpg",
		 Elements.SIDE_ALL, 4.0, 2.5);

		Material shading = new Material();
		cone.setShading(shading);	 
		
		getWorld().addSceneNode(cone);

		// build world light
		OrbitGroup sun = new OrbitGroup(new TestLight(
		 new DirectionalLight()));
		sun.initActuation(new Vector4d(
		 -Math.PI/6, -Math.PI/8, 0, 0));
		
		getWorld().addSceneNode(sun);
		
		// done
		showFrame();
	}
			
	// personal body ============================================

}