import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import j3dui.utils.Debug;
import j3dui.utils.*;
import j3dui.utils.app.*;
import j3dui.utils.objects.*;
import j3dui.utils.blocks.*;
import j3dui.control.*;
import j3dui.control.inputs.*;
import j3dui.control.mappers.*;
import j3dui.control.mappers.intuitive.*;
import j3dui.control.actuators.*;
import j3dui.control.actuators.groups.*;

/**
Example demonstrating WRM (world-relative) intuitive mapping.
<P>
Refer to the book for further details about this example.

@author Jon Barrilleax,
copyright (c) 1999 Jon Barrilleaux,
All Rights Reserved.
*/

public class WrmMapping extends SingleFrame {
	
	public static void main(String[] argv) {
		Debug.loadAllProperties();
		new WrmMapping();
	}
	
	// public interface =========================================
	
	public WrmMapping() {
		setTitle("Example - WrmMapping");
		
		new HelpDialog(this, "Help - WrmMapping",
		 "Example demonstrating WRM (world-relative) " +
		 "intuitive control mapping." +
		 "\n" +
		 "\n" +
		 "> Left-button-shift drag or arrow keys-shift to " +
		 "tilt and turn view." +
		 "\n" +
		 "> Left-button drag red thing to slide it with " +
		 "true WRM." +
		 "\n" +
		 "> Left-button drag green thing to slide it with " +
		 "quasi-WRM." +
		 "\n" +
		 "> Left-button drag blue thing to slide it with " +
		 "pseudo-WRM." +
		 "\n" +
		 "> Left-button-ctrl drag things to roll them using " +
		 "respective form of WRM." +
		 "\n" +
		 "> Use arrow keys with mouse over thing to " +
		 "manipulate it." +
		 "\n" +
		 "> Escape key to cancel manipulation." +
		 "\n" +
		 "\n" +
		 ">>> Tilt and turn view, then repeat manipulations." +
		 "\n" +
		 "\n" +
		 "Refer to the book for further details about this " +
		 "example." +
		 "\n" +
		 "copyright (c) 1999 Jon Barrilleaux, " +
		 "All Rights Reserved."
		);
		
		// setup orbitable view, first + shift
		getWorld().addViewNode(
		 ActuationBlocks.buildViewOrbiter(getView(),
		 new Vector3d(-Math.PI/8, Math.PI/16, 0),
		 11, Input.BUTTON_FIRST, Input.MODIFIER_SHIFT,
		 Input.MODIFIER_SHIFT)
		);
		
		// setup manipulation targets
		/// WRM target
		AxisAngleSphereGroup sphereWrm =
		 new AxisAngleSphereGroup(new TestThing(
		 TestThing.BOX_TRANSLATION, TestThing.BALL_SPHERE));
		AffineGroup affineWrm = new AffineGroup(sphereWrm);
		affineWrm.getTranslation().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(-2.5, 2.5));

		//// use update not init since hit is in source space		 
		affineWrm.getTranslation().updateActuation(
		 new Vector4d(-2.5, 0, 0, 0));
		getWorld().addSceneNode(affineWrm);

		EnableSplitter splitterWrm = new EnableSplitter();
		splitterWrm.addEventTarget(affineWrm);
		splitterWrm.addEventTarget(sphereWrm);
		
		/// Quasi-WRM target
		AxisAngleSphereGroup sphereQWrm =
		 new AxisAngleSphereGroup(new TestThing(
		 TestThing.BOX_ROTATION, TestThing.BALL_SPHERE));
		AffineGroup affineQWrm = new AffineGroup(sphereQWrm);
		affineQWrm.getTranslation().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(-2.5, 2.5));
		 
		//// use update not init since hit is in source space		 
		affineQWrm.getTranslation().updateActuation(
		 new Vector4d(0, 0, -2.5, 0));
		getWorld().addSceneNode(affineQWrm);

		EnableSplitter splitterQWrm = new EnableSplitter();
		splitterQWrm.addEventTarget(affineQWrm);
		splitterQWrm.addEventTarget(sphereQWrm);
		
		/// Pseudo-WRM target
		AxisAngleSphereGroup spherePWrm =
		 new AxisAngleSphereGroup(new TestThing(
		 TestThing.BOX_SCALE, TestThing.BALL_SPHERE));
		AffineGroup affinePWrm = new AffineGroup(spherePWrm);
		affinePWrm.getTranslation().getPlugin().setTargetClamp(
		 Mapper.DIM_ALL, new Vector2d(-2.5, 2.5));
		 
		//// use init not update since hit is in target space		 
		affinePWrm.getTranslation().initActuation(
		 new Vector4d(0, -2, 1.5, 0));
		getWorld().addSceneNode(affinePWrm);

		EnableSplitter splitterPWrm = new EnableSplitter();
		splitterPWrm.addEventTarget(affinePWrm);
		splitterPWrm.addEventTarget(spherePWrm);
		
		// setup WRM
		AffineGroup affine;
		
		/// build source plane X
		TestPlane planeX = new TestPlane(
		 TestPlane.PLANE_DEFAULT, 5, 5);
		 
		affine = new AffineGroup(planeX);
		affine.getTranslation().initActuation(
		 new Vector4d(-2.5, 0, 0, 0));
		affine.getAxisAngle().initActuation(
		 new Vector4d(0, 1, 0, Math.PI/2.0));
		getWorld().addSceneNode(affine);

		/// build source plane Y
		TestPlane planeY = new TestPlane(
		 TestPlane.PLANE_DEFAULT, 5, 5);
		
		affine = new AffineGroup(planeY);
		affine.getTranslation().initActuation(
		 new Vector4d(0, -2.5, 0, 0));
		affine.getAxisAngle().initActuation(
		 new Vector4d(1, 0, 0, -Math.PI/2.0));
		getWorld().addSceneNode(affine);

		/// build source plane Z
		TestPlane planeZ = new TestPlane(
		 TestPlane.PLANE_DEFAULT, 5, 5);
		
		affine = new AffineGroup(planeZ);
		affine.getTranslation().initActuation(
		 new Vector4d(0, 0, -2.5, 0));
		affine.getAxisAngle().initActuation(
		 new Vector4d(1, 0, 0, 0));
		getWorld().addSceneNode(affine);

		/// build source picker
		ArrayList wrmSourceList = new ArrayList();
		wrmSourceList.add(planeX);
		wrmSourceList.add(planeY);
		wrmSourceList.add(planeZ);
			
		PickEngine wrmSourcePicker = new PickEngine(
		 getWorld().getSceneRoot(), wrmSourceList);
		wrmSourcePicker.setHitAll(true);

		// setup manipulation controls
		/// translation draggers, first button
		InputDragSplitter absXlate = new InputDragSplitter();
		
		ActuationBlocks.buildAbsoluteDragger(absXlate,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_NONE, Input.MODIFIER_NONE);
		
		/// rotation dragger, first button + control
		InputDragSplitter absRotate = new InputDragSplitter();
		
		ActuationBlocks.buildAbsoluteDragger(absRotate,
		 getView(), Input.BUTTON_FIRST,
		 Input.MODIFIER_CTRL, Input.MODIFIER_CTRL);
		
		/// manipulation mappers
		InputDragTarget mapper;

		//// WRM: world translation, source rotation 
		mapper = IntuitiveBlocks.buildWrmTranslationMapper(
		 affineWrm, new WrmDragPlugin(null, wrmSourcePicker),
		 false, false);
		absXlate.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildWrmSphereMapper(
		 sphereWrm, new WrmDragPlugin(wrmSourcePicker),
		 true, true);
		absRotate.addEventTarget(mapper);
		
		//// Quasi-WRM: world translation, source rotation
		mapper = IntuitiveBlocks.buildWrmTranslationMapper(
		 affineQWrm, new QuasiWrmDragPlugin(null,
		 wrmSourcePicker), false, false);
		absXlate.addEventTarget(mapper);
		
		mapper = IntuitiveBlocks.buildWrmSphereMapper(
		 sphereQWrm, new QuasiWrmDragPlugin(wrmSourcePicker),
		 true, true);
		absRotate.addEventTarget(mapper);

		//// Pseudo-WRM: picked translation, fixed rotation
		mapper = IntuitiveBlocks.buildStickyWrmTranslationMapper(
		 planeY, affinePWrm, getWorld().getSceneRoot(),
		 true, true);
		absXlate.addEventTarget(mapper);

		mapper = IntuitiveBlocks.buildWrmSphereMapper(
		 spherePWrm, new PseudoWrmDragPlugin(planeY,
		 affinePWrm), true, true);
		absRotate.addEventTarget(mapper);
		 
		// setup over management
		ArrayList pickList, enableList;
		
		pickList = new ArrayList();
		pickList.add(affineWrm);
		pickList.add(affineQWrm);
		pickList.add(affinePWrm);
		 
		enableList = new ArrayList();
		enableList.add(splitterWrm);
		enableList.add(splitterQWrm);
		enableList.add(splitterPWrm);
		
		IntuitiveBlocks.buildOverEnabler(getView(),
		 getWorld().getSceneRoot(), pickList, enableList);

		// done
		showFrame();
	}
			
	// personal body ============================================

}